(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Laura Lowenthal, projet Protheo, INRIA Lorraine           *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: genr_testing.ml,v 1.12 2008/01/17 14:31:41 lowenthl Exp $ *)

(** {Generation of testing expressions for a moca type declaration.} *)
open Funutils
open Parsetree
open Genr_base
open Tgenr_base
open Genr_equalities
;;

let test_module_name = "Gentest"
let global_test_fun_name = "testing"
let test_fun_name = "testi"
;;

let global_test_fun module_name =
  let global_test_fun_ident = Longident.Lident global_test_fun_name in
  let module_string = mk_string (module_name ^ " (automatic)") in
  Code.Pstr_eval (Code.apply1 global_test_fun_ident module_string)

let eval_test i exp =
  let test_fun_ident = Longident.Lident test_fun_name in
  Code.Pstr_eval (Code.apply2 test_fun_ident (mk_int i) exp)
;;

let genr_test_preamble module_name = [
  Code.Pstr_open (Longident.Lident module_name);
  Code.Pstr_open (Longident.Lident test_module_name);
  global_test_fun module_name
]
;;

let genr_testing rnd num_tests val_depth (_, td) =
  match td.ptype_kind, td.ptype_private with
  (* We only generate tests for private sum types. *)
  | Ptype_record _, Asttypes.Public
  | Ptype_abstract _, Asttypes.Public -> []
  | Ptype_variant _, Asttypes.Public
  | Ptype_record _, Asttypes.Private
  | Ptype_abstract _, Asttypes.Private -> []
  | Ptype_variant cdefs, Asttypes.Private ->
    let f_gen_values ct n =
      Genr_values.genr_core_values rnd n val_depth ct in
    Listutils.flat_map
      (genr_values_and_equalities num_tests f_gen_values) cdefs
;;

let genr_test_body rnd num_tests val_depth ntds =
  let exps =
    Listutils.flat_map (genr_testing rnd num_tests val_depth) ntds in
  let test_numbers =
    Listutils.from_to 0 (List.length exps - 1) succ in
  List.map2 eval_test test_numbers exps
;;

let genr_test module_name rnd num_tests val_depth ntds =
  genr_test_preamble module_name @
  genr_test_body rnd num_tests val_depth ntds
;;

let genr_and_print_test ob module_name rnd num_tests val_depth ntds =
  Random.self_init ();
  let structure_items =
    genr_test module_name rnd num_tests val_depth ntds in
  Pr_code.structure ob structure_items
;;
